class E393_Port {

  constructor(options){
    let defaults = {
      index: 0,
      sensor: ["none","none","none","none"],
      mux: "none",
      parameters: {
        wb_en:          0,
        autoexp_on:     0,
        compressor_run: 0,
        sensor_run:     0,
        color:          0,
        quality:        0,
        expos:          0,
        woi_width:      0,
        woi_height:     0,
        trig:           0,
        trig_master:    0,
        trig_period:    0,
        trig_condition: 0,
        trig_out:       0,
        gainr:          0,
        gaing:          0,
        gainb:          0,
        gaingb:         0
      },
      timestamps: []
    };

    if ($.isXMLDoc(options)){

      this._data        = $.extend(defaults,{});
      this._data.index  = parseInt($(options).attr("index"));
      this._data.sensor = $(options).attr("sensor").split(",").map(v => v.trim());

      let self = this;

      let pars = $(options).find("parameters");

      $(pars).children().each(function(){
        let name  = this.nodeName;
        let value = parseInt($(this).text());
        self._data.parameters[name] = value;
      });

      let ts = $(options).find("timestamps");
      /*
      $(ts).children().each(function(){
        self._data.timestamps.push({
          frame: parseInt($(this).attr("frame")),
          ts:    parseFloat($(this).attr("ts")),
          ptr:   parseInt($(this).attr("ptr")),
        });
      });
      */

      this.update_timestamps(ts);
      //this._data.parameters.wb_en      = parseInt($(pars).find("wb_en"));
      //this._data.parameters.autoexp_on = parseInt($(pars).find("autoexp_on"));

    }else{
      this._data = $.extend(defaults,options);
    }

    this.index       = this._data.index;
    this.sensor      = this._data.sensor;
    this.mux         = this._data.mux;
    this.parameters  = this._data.parameters;
    this.timestamps  = this._data.timestamps;
  }

  // update timestamps
  update_timestamps(ts){

    let self = this;
    this._data.timestamps = [];

    $(ts).children().each(function(){
      self._data.timestamps.push({
        frame: parseInt($(this).attr("frame")),
        ts:    parseFloat($(this).attr("ts")),
        ptr:   parseInt($(this).attr("ptr")),
      });
    });

  }

}

class E393 {

  // options can be an xmldoc
  constructor(options){

    let defaults = {
      master_port: "N/A",
      systime: "",
      systimestamp: 0,
      uptime: "",
      temperature: {
        cpu: 0,
        b10389: 0,
        sda: 0,
        sdb: 0
      },
      storage: "",
      recorder: "",
      gps: "",
      ports: [
        new E393_Port({index:0}),
        new E393_Port({index:1}),
        new E393_Port({index:2}),
        new E393_Port({index:3})
      ]
    }

    if ($.isXMLDoc(options)){

      this._data = $.extend(defaults,{});

      this._data.master_port  = parseInt($(options).find("master_port").text());
      this._data.systime      =          $(options).find("systime").text();
      this._data.systimestamp =          $(options).find("systimestamp").text();
      this._data.uptime       =          $(options).find("uptime").text();
      this._data.gps          =          $(options).find("gps").text().trim();
      this._data.storage      =          $(options).find("storage").text().trim();
      this._data.recorder     =          $(options).find("recorder").text().trim();

      this._data.temperature.cpu    = parseFloat($(options).find("temperature").find("cpu").text());
      this._data.temperature.b10389 = parseFloat($(options).find("temperature").find("b10389").text());
      this._data.temperature.sda    = parseFloat($(options).find("temperature").find("sda").text());
      this._data.temperature.sdb    = parseFloat($(options).find("temperature").find("sdb").text());

      let self = this;

      $(options).find("port").each(function(){
        let index = parseInt($(this).attr("index"));
        self._data.ports[index] = new E393_Port(this);
      });

    }else{
      this._data = $.extend(defaults,options);
    }

    this.master_port = this._data.master_port;
    this.systime = this._data.systime;
    this.systimestamp = this._data.systimestamp;
    this.uptime = this._data.uptime;
    this.temperature = this._data.temperature;
    this.storage = this._data.storage;
    this.recorder = this._data.recorder;
    this.gps = this._data.gps;
    this.ports = this._data.ports;
  }

}
