// libs
const url     = require('url');
const express = require('express');
const fs      = require('fs');
const path    = require('path');
// more libs
const router = express.Router();
const log = console.log;
const rclnodejs = require('rclnodejs');

let ros2node = {};

rclnodejs.init().then(() => {
  const mac = get_my_mac();
  ros2node = rclnodejs.createNode('_nodejs_client_'+mac);
  rclnodejs.spin(ros2node);
  log('rclnodejs initialized');
});

/* GET home page. */
router.get('/', function(req, res, next) {
  res.render('index');
});

const { execSync } = require('child_process');

router.get('/api/nodelist', function(req, res, next) {
  // shell command
  const stdout = execSync('ros2 node list');
  const uri = url.parse(req.url).pathname;
  res.send(stdout);
});

router.post('/api/*', function(req, res, next) {

  const master  = req.body.master;
  const targets = req.body.targets;
  const cmd     = req.body.cmd;

  // handle errors
  if ((master==='')||(master==='-')) {
    res.send("Invalid master node name");
    return;
  }

  const client = ros2node.createClient('elphel_interfaces/srv/StrReqStrRes', master+'/cmd');
  const request = {
    request: [
      '<document>',
        '<targets>'+targets+'</targets>',
        '<cmd>'+cmd+'</cmd>',
      '</document>'].join('')
  };

  console.log(`Sending: ${typeof request}`, request);
  client.sendRequest(request, (response) => {
    console.log(`Result: ${typeof response}`, response);
    //rclnodejs.shutdown();
    res.send(response.response);
  });

  //rclnodejs.spin(node);

});

module.exports = router;

function get_my_mac(last_three_octets=true){
  
  let tmp_mac = "000000000000";
  let dir = "/sys/class/net/";
  
  fs.readdirSync(dir).forEach(file => {
    //log(file);
    let p = /^(eth|enp|wlp)/gi;
    let a = p.test(file);
    if(a){
      tmp_mac = fs.readFileSync(path.join(dir,file,'address'),'utf-8').trim();
      tmp_mac = tmp_mac.replace(/:/g,'');
    }
    //log(a);
  });
  
  if (last_three_octets) {
    tmp_mac = tmp_mac.slice(-6);
  }

  return tmp_mac;
  
}
